//allows access to external flash/ram/rom using table reads and writes

//eg:	x=romuint(0xffff) sets x to contents of 0xffff in rom (external flash)
//		addr=0xffff;x=romuint(addr) does same thing
//		x=ramuint(0x1fe) sets x to contents of 0x1fe in internal ram
//		addr=0x1fe;x=ramuint(addr) does same thing
//		x=ramuchar_pointedto_contents(0x1ff), if romuchar(0x1ff)=0x100
//			sets x to whatever is at ram location 0x100

#define romuint(x)													(*((rom unsigned int *) (x)))
#define ramuint(x)													(*((unsigned int *) (x)))
#define ramuint_pointedto_contents(x)				ramuint(ramuint(x))
#define ramuchar(x)													(*((unsigned char *) (x)))
#define ramuchar_pointedto_contents(x)			ramuchar(ramuchar(x))

//this is where the entry point is stored in flash
//it should be the top of the imaginary rom space
#define	ENTRY_POINT_STORAGE_IN_FLASH		0x1fff

void SetupFlash(void);
void EraseFlash(void);
void EraseSector(unsigned int sector);
#define ReadFlash(x)	romuint(x)
unsigned char WriteFlash(unsigned int addr, unsigned int data);
unsigned char WriteFlashArray(unsigned int rom_addr, unsigned int *ram_addr, unsigned char length);
void ReadFlashArray(unsigned int rom_addr, unsigned int *ram_addr, unsigned char length);
unsigned char TestFlashBrief(void);
unsigned char TestFlashThorough(void);
unsigned int FlashManufacturerID(void);
unsigned int FlashDeviceID(void);

//flash status
unsigned char FlashStatus(void);
//#defines for flash status
#define STATUS_READY		0
#define STATUS_ERSUSP		1
#define STATUS_TIMEOUT	2
#define STATUS_BUSY			3
#define STATUS_ERROR		4

void api_flash_device_id(void);
void api_flash_manufacturer_id(void);
void api_erase_sector(void);
void api_write_flash(void);
void api_read_flash(void);
void api_write_flash_array(void);
void api_read_flash_array(void);
void api_test_flash(void);

//minimum address that flash routines may write to
//	0x500 for PIC17C42
//	0x1000 for PIC17C43
//	0x2000 for PIC17C44
#define	FLASH_MIN_ADDRESS	0x1000

#define	FLASH_A16_DIR		DDRBbits.RB5
#define	FLASH_A16				PORTBbits.RB5

//put api_comms_data and control at addresses 0xFC to 0xFF in second bank
#define api_flash_uint_control 		ramuint(0x1FE)

//codesto put in api_flash_uint_control
#define API_REQ_NOTHING									0xed00
#define API_REQ_FLASH_CODE_UPDATE				0xed01
#define API_REQ_FLASH_READ_WORD					0xed02
#define API_REQ_FLASH_WRITE_WORD				0xed03
#define API_REQ_FLASH_ERASE_SECTOR			0xed04
#define API_REQ_FLASH_MANU_ID						0xed05
#define API_REQ_FLASH_DEVICE_ID					0xed06
#define API_REQ_FLASH_WRITE_WORD_ARRAY	0xed07
#define API_REQ_FLASH_READ_WORD_ARRAY		0xed08
#define	API_REQ_FLASH_RESET							0xed09


//data for api_flash_uint_control calls to use
#define api_flash_uint_address			ramuint(0x1FC)			//usually points to rom
#define api_flash_uint_data 				ramuint(0x1FA)
#define api_flash_uint_hi_data			ramuchar(0x1FA+1)
#define api_flash_uint_lo_data			ramuchar(0x1FA+0)
#define api_flash_uint_working 			ramuint(0x1F8)
#define api_flash_uint_hi_working		ramuchar(0x1F8+1)
#define api_flash_uint_lo_working		ramuchar(0x1F8+0)
#define api_flash_uchar_working 		ramuchar(0x1F7)
#define api_flash_uchar_return 			ramuchar(0x1F6)


#define	SelectFlashBank(x);		FLASH_A16=x;

#define api_erase_flash()		EraseFlash()

//to check if flash ready, do two successive reads of any address
//	and check to see if pins toggle, using XOR
#define flash_busy		!(romuint(0xeeee) & romuint(0xeeee))

//wait for flash to erase
//wait for flash to erase
#define wait_for_flash_finish; 					\
																				\
do 																			\
{																				\
	if(romuint(0x8080) & 0x0020)					\
	Nop();																\
	Nop();																\
} while(!(romuint(0x8080) & 0x0080));		

//must reset flash between successive manufacturer/device reads
#define	reset_flash;  									\
romuint(0xaaaa)=0xf0f0;									\
romuint(0xaaaa)=0xf0f0;									\
Delay10Us(4);														

#define ResetFlash()	reset_flash

#define	return_after_lcall_to_code_in_flash		\
_asm																					\
RETURN																				\
_endasm								

#define	lcall_routines_for_flash_in_bootup_code		\
_asm																							\
MOVLW 0x00																				\
MOVPF WREG, PCLATH																\
LCALL 0x00																				\
_endasm		
